#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2021 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import re
import sys, os
import random
import copy

from . import TloggCfg, TloggGlbl

from TermTk import *

def filtersFormLayout(win):
    '''
    This form is inspired by glogg "Filters..." form
    '''

    filters = copy.deepcopy(TloggCfg.filters)

    leftRightLayout = TTkHBoxLayout()
    leftLayout        = TTkGridLayout()
    rightLayout       = TTkGridLayout()
    bottomRightLayout = TTkGridLayout()
    leftRightLayout.addItem(leftLayout)
    leftRightLayout.addItem(rightLayout)

    frameFilters = TTkFrame(border=True, layout=TTkGridLayout())
    leftLayout.addWidget(frameFilters,0,0,1,5)

    listFilters = TTkList(parent=frameFilters)

    addButton    = TTkButton(text="+",maxWidth=3)
    removeButton = TTkButton(text="-",maxWidth=3)
    upButton     = TTkButton(text="▲",maxWidth=3)
    downButton   = TTkButton(text="▼",maxWidth=3)

    leftLayout.addWidget(addButton       ,1,0)
    leftLayout.addWidget(removeButton    ,1,1)
    leftLayout.addWidget(TTkSpacer() ,1,2)
    leftLayout.addWidget(upButton        ,1,3)
    leftLayout.addWidget(downButton      ,1,4)

    rightLayout.addWidget(TTkLabel(text="Pattern:"),0,0)
    rightLayout.addWidget(pattern:=TTkLineEdit(text="-----"),0,1)
    rightLayout.addWidget(TTkLabel(text="Ignore Case:"),1,0)
    rightLayout.addWidget(ignoreCase:=TTkCheckbox(),1,1)
    rightLayout.addWidget(TTkLabel(text="FG Color:"),2,0)
    rightLayout.addWidget(fgColor := TTkColorButtonPicker(border=False, color=TTkColor.fg('#eeeeee') ),2,1)
    rightLayout.addWidget(TTkLabel(text="BG Color:"),3,0)
    rightLayout.addWidget(bgColor := TTkColorButtonPicker(border=False, color=TTkColor.bg('#333333') ),3,1)
    rightLayout.addWidget(TTkSpacer() ,4,0,1,2)

    rightLayout.addItem(bottomRightLayout ,5,0,1,2)
    bottomRightLayout.addWidget(applyBtn  := TTkButton(text="Apply",  border=True, maxHeight=3),0,1)
    bottomRightLayout.addWidget(cancelBtn := TTkButton(text="Cancel", border=True, maxHeight=3),0,2)
    bottomRightLayout.addWidget(okBtn     := TTkButton(text="OK",     border=True, maxHeight=3),0,3)

    def _move(offset):
        def _moveUpDown():
            if items := listFilters.selectedItems():
                item = items[0]
                index = listFilters.indexOf(item)
                listFilters.moveItem(index,index+offset)
        return _moveUpDown
    upButton.clicked.connect(_move(-1))
    downButton.clicked.connect(_move(1))

    def _addCallback():
        filter = {'pattern':"<PATTERN>", 'ignorecase':True, 'fg':"#FFFFFF", 'bg':"#000000" }
        filters.append(filter)
        listFilters.addItem(item=filter['pattern'],data=filter)
    addButton.clicked.connect(_addCallback)

    def _removeCallback():
        # Clear all the signals
        pattern.textEdited.clear()
        ignoreCase.clicked.clear()
        fgColor.colorSelected.clear()
        bgColor.colorSelected.clear()
        if items := listFilters.selectedItems():
            filters.remove(items[0].data)
            listFilters.removeItem(items[0])
    removeButton.clicked.connect(_removeCallback)

    def _saveFilters():
        filters = []
        for item in listFilters.items():
            filters.append(item.data)
        TloggCfg.filters = filters
        TloggCfg.save(searches=False,filters=True)
        TloggGlbl.refreshViews()
        # TTkHelper.updateAll()

    applyBtn.clicked.connect(_saveFilters)
    okBtn.clicked.connect(_saveFilters)
    okBtn.clicked.connect(win.close)
    cancelBtn.clicked.connect(win.close)

    @ttk.pyTTkSlot(TTkWidget)
    def _listCallback(item):
        if filter:=item.data:
            # Clear all the signals
            pattern.textEdited.clear()
            ignoreCase.clicked.clear()
            fgColor.colorSelected.clear()
            bgColor.colorSelected.clear()
            # Config the filter widgets
            pattern.setText(filter['pattern'])
            ignoreCase.setCheckState(TTkK.Checked if filter['ignorecase'] else TTkK.Unchecked)
            fgColor.setColor(TTkColor.bg(filter['fg']))
            bgColor.setColor(TTkColor.bg(filter['bg']))
            # Connect the actions
            ## Pattern Line Edit
            def _setPattern(p):
                item.text=p
                filter['pattern'] = p
            pattern.textEdited.connect(_setPattern)
            ## Case Sensitivity checkbox
            def _setCase(c):filter['ignorecase'] = c
            ignoreCase.clicked.connect(_setCase)
            ## Color Button
            def _setFg(c):filter['fg'] = c.getHex(TTkK.Background)
            def _setBg(c):filter['bg'] = c.getHex(TTkK.Background)
            fgColor.colorSelected.connect(_setFg)
            bgColor.colorSelected.connect(_setBg)


    listFilters.itemClicked.connect(_listCallback)

    for i,filter in enumerate(filters):
        # ali = TTkAbstractListItem(text=filter['pattern'],data=filter)
        listFilters.addItem(item=filter['pattern'],data=filter)

    return leftRightLayout

def preferencesForm(root=None):
    preferencesLayout = TTkGridLayout(columnMinWidth=1)
    frame = TTkFrame(parent=root, layout=preferencesLayout, border=0)

    frameFilters = TTkFrame(border=True, title="Filters...", layout=filtersFormLayout())
    preferencesLayout.addWidget(frameFilters,0,0)

    return frame


#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2021 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from TermTk.TTkCore.log import TTkLog
from TermTk.TTkCore.color import TTkColor
from TermTk import TTkAbout, TTkWindow
from .cfg import TloggCfg

class About(TTkAbout):
    tlogg = [
        " __    ___                               ",
        "/\ \__/\_ \                              ",
        "\ \ ,_\//\ \     ___      __      __     ",
        " \ \ \/ \ \ \   / __`\  /'_ `\  /'_ `\   ",
        "  \ \ \_ \_\ \_/\ \L\ \/\ \L\ \/\ \L\ \  ",
        "   \ \__\/\____\ \____/\ \____ \ \____ \ ",
        "    \/__/\/____/\/___/  \/___L\ \/___L\ \\",
        "                          /\____/ /\____/",
        "                          \_/__/  \_/__/ "]

    __slots__=('_image')
    def __init__(self, *args, **kwargs):
        TTkAbout.__init__(self, *args, **kwargs)
        self._name = kwargs.get('name' , 'About' )
        self.setTitle('[PierCecco Cecco] Eugenio Parodi proudly presents...')
        self.resize(56,16)



    def paintEvent(self):
        c = [0xFF,0xFF,0xFF]
        for y, line in enumerate(About.tlogg):
            self._canvas.drawText(pos=(13,3+y),text=line, color=TTkColor.fg(f'#{c[0]:02X}{c[1]:02X}{c[2]:02X}'))
            c[2]-=0x18
            c[0]-=0x08
        self._canvas.drawText(pos=(26,4),text=f"  Version: {TloggCfg.version}", color=TTkColor.fg('#AAAAFF'))
        self._canvas.drawText(pos=(14,11),text=f"Powered By, pyTermTk")
        self._canvas.drawText(pos=(2,13),text=f"https://github.com/ceccopierangiolieugenio/tlogg", color=TTkColor.fg('#44FFFF'))
        self._canvas.drawText(pos=(2,14),text=f"https://github.com/ceccopierangiolieugenio/pyTermTk", color=TTkColor.fg('#44FFFF'))

        TTkWindow.paintEvent(self)



#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2021 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import yaml

class TloggCfg:
    version="0.2.0a1"
    name="tlogg"
    pathCfg="."
    filters=[]
    maxsearches=200
    searches=[]

    @staticmethod
    def save(searches=True,filters=True):
        os.makedirs(TloggCfg.pathCfg, exist_ok=True)
        filtersPath  = os.path.join(TloggCfg.pathCfg,'filters.yaml')
        searchesPath = os.path.join(TloggCfg.pathCfg,'searches.yaml')
        if filters:
            with open(filtersPath, 'w') as f:
                yaml.dump(TloggCfg.filters, f, sort_keys=False, default_flow_style=False)
        if searches:
            with open(searchesPath, 'w') as f:
                yaml.dump(TloggCfg.searches, f, sort_keys=False, default_flow_style=False)

    @staticmethod
    def load():
        filtersPath = os.path.join(TloggCfg.pathCfg,'filters.yaml')
        searchesPath = os.path.join(TloggCfg.pathCfg,'searches.yaml')
        if not os.path.exists(filtersPath): return
        with open(filtersPath) as f:
            TloggCfg.filters = yaml.load(f, Loader=yaml.SafeLoader)
        if not os.path.exists(searchesPath): return
        with open(searchesPath) as f:
            TloggCfg.searches = yaml.load(f, Loader=yaml.SafeLoader)



#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2021 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import re
import sys
import argparse

import appdirs

from TermTk import *

from TermTk import TTk, TTkK, TTkLog, TTkColor
from TermTk import pyTTkSlot, pyTTkSignal
from TermTk import TTkFrame
from TermTk import TColor, TText
from TermTk import TTkAbstractScrollArea, TTkAbstractScrollView

from .cfg  import *
from .glbl import *
from .about import *
from .fileviewer  import *
from .preferences import *

def main():
    TloggCfg.pathCfg = appdirs.user_config_dir("tlogg")

    parser = argparse.ArgumentParser()
    # parser.add_argument('-f', help='Full Screen', action='store_true')
    parser.add_argument('-c', help=f'config folder (default: "{TloggCfg.pathCfg}")', default=TloggCfg.pathCfg)
    parser.add_argument('filename', type=str, nargs='+',
                    help='the filename/s')
    args = parser.parse_args()

    TTkLog.use_default_file_logging()

    TloggCfg.pathCfg = args.c
    TTkLog.debug(f"Config Path: {TloggCfg.pathCfg}")

    TloggCfg.load()

    root = TTk(layout=TTkGridLayout())
    splitter = TTkSplitter(parent=root, orientation=TTkK.VERTICAL)
    tab = TTkTabWidget(parent=splitter, border=False)
    # tab = TTkTabWidget(parent=splitter, border=True)
    splitter.addWidget(TTkLogViewer(),3)


    fileMenu = tab.addMenu("[&File]")
    buttonOpen    = fileMenu.addMenu("Open")
    buttonClose   = fileMenu.addMenu("Close")
    fileMenu.addSpacer()
    buttonFilters = fileMenu.addMenu("&Filters...")
    fileMenu.addSpacer()
    buttonAbout = fileMenu.addMenu("&About...")
    fileMenu.addSpacer()
    buttonExit    = fileMenu.addMenu("Exit")
    buttonExit.menuButtonClicked.connect(lambda _: root.quit())

    def showFilters(btn):
        win = TTkWindow(title="Filters...", size=(70,20), border=True)
        win.setLayout(filtersFormLayout(win))
        TTkHelper.overlay(buttonFilters, win, 0,0)
    buttonFilters.menuButtonClicked.connect(showFilters)

    def showAbout(btn):
        TTkHelper.overlay(buttonFilters, About(), 0,0)
    buttonAbout.menuButtonClicked.connect(showAbout)

    # tab.addTab(preferencesForm(), "-Setup-")

    for file in args.filename:
        tabSplitter = TTkSplitter(orientation=TTkK.VERTICAL)
        tab.addTab(tabSplitter, file)
        topFrame    = TTkFrame(parent=tabSplitter, border=False, layout=TTkVBoxLayout())
        bottomFrame = TTkFrame(parent=tabSplitter, border=False, layout=TTkVBoxLayout())


        # Define the bottom layout widgets
        bottomLayoutSearch = TTkHBoxLayout()
        bls_label_1  = TTkLabel(text=" Text:", maxWidth=6)
        bls_label_2  = TTkLabel(text="Ignore case:", maxWidth=12)
        bls_cb_icase = TTkCheckbox(maxWidth=3)
        bls_search   = TTkButton(text="Search", maxWidth=10)
        bls_searchbox  = TTkComboBox(editable=True)
        bls_searchbox.addItems(TloggCfg.searches)
        bls_searchbox.setCurrentIndex(0)

        bottomLayoutSearch.addWidget(bls_label_2)
        bottomLayoutSearch.addWidget(bls_cb_icase)
        bottomLayoutSearch.addWidget(bls_label_1)
        bottomLayoutSearch.addWidget(bls_searchbox)
        bottomLayoutSearch.addWidget(bls_search)

        bottomFrame.layout().addItem(bottomLayoutSearch)

        # Define the main file Viewer
        fileBuffer = TTkFileBuffer(file, 0x1000, 0x10)
        topViewport = FileViewer(filebuffer=fileBuffer)
        topViewer = FileViewerArea(parent=topFrame, fileView=topViewport)
        fileBuffer.indexUpdated.connect(topViewport.fileIndexing)
        fileBuffer.indexed.connect(topViewport.fileIndexed)
        # Define the Search Viewer
        bottomViewport = FileViewerSearch(filebuffer=fileBuffer)
        bottomViewer = FileViewerArea(parent=bottomFrame, fileView=bottomViewport)
        bottomViewport.selected.connect(topViewport.selectAndMove)
        bottomViewport.marked.connect(topViewport.markIndexes)
        topViewport.marked.connect(bottomViewport.markIndexes)

        # Add those viewpoers to the global list to allow dynamic refresh
        # TODO: Try to get rid of this
        TloggGlbl.addRefView(topViewport)
        TloggGlbl.addRefView(bottomViewport)

        def _search(_=None):
            searchtext = bls_searchbox.currentText()
            TTkLog.debug(f"{searchtext=}")
            indexes = fileBuffer.searchRe(searchtext, ignoreCase=bls_cb_icase.checkState() == TTkK.Checked)
            bottomViewport.searchedIndexes(indexes)
            topViewport.searchedIndexes(indexes)
            if TloggCfg.searches:
                x = set(TloggCfg.searches)
                TTkLog.debug(f"{x}")
                TloggCfg.searches = list(x)
                if searchtext in TloggCfg.searches:
                    TloggCfg.searches.remove(searchtext)
            TloggCfg.searches.insert(0, searchtext)
            TloggCfg.searches = TloggCfg.searches[:TloggCfg.maxsearches]
            TloggCfg.save(searches=True,filters=False)
            bls_searchbox.clear()
            bls_searchbox.addItems(TloggCfg.searches)
            bls_searchbox.setCurrentIndex(0)

        bls_search.clicked.connect(_search)
        bls_searchbox.editTextChanged.connect(_search)

    root.mainloop()



#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2021 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import re
import sys
import argparse

from TermTk import *

from TermTk import TTkK
from TermTk import TTkLog
from TermTk import TTkColor
from TermTk import TTk
from TermTk import TTkFileBuffer
from TermTk import TTkFrame
from TermTk import TColor
from TermTk import TText
from TermTk import pyTTkSlot, pyTTkSignal
from TermTk import TTkAbstractScrollArea
from TermTk import TTkAbstractScrollView

from . import TloggCfg

class FileViewer(TTkAbstractScrollView):
    __slots__ = (
        '_fileBuffer', '_indexesMark', '_indexesSearched', '_selected', '_indexing',
        # Signals
        'selected', 'marked')
    def __init__(self, *args, **kwargs):
        self._indexesMark = []
        self._indexesSearched = []
        self._indexing = None
        self._selected = -1
        # Signals
        self.selected = pyTTkSignal(int)
        self.marked = pyTTkSignal(list)
        TTkAbstractScrollView.__init__(self, *args, **kwargs)
        self._name = kwargs.get('name' , 'FileViewer' )
        self._fileBuffer = kwargs.get('filebuffer')
        self.viewChanged.connect(self._viewChangedHandler)

    @pyTTkSlot()
    def _viewChangedHandler(self):
        self.update()

    @pyTTkSlot(float)
    def fileIndexing(self, percentage):
        self._indexing = percentage
        self.viewChanged.emit()

    @pyTTkSlot()
    def fileIndexed(self):
        self._indexing = None
        self.viewChanged.emit()

    def markIndexes(self, indexes):
        self._indexesMark = indexes
        self.viewChanged.emit()

    def searchedIndexes(self, indexes):
        self._indexesSearched = indexes
        self.viewChanged.emit()

    def viewFullAreaSize(self) -> (int, int):
        w = 2+self._fileBuffer.getWidth()
        h = self._fileBuffer.getLen()
        return w , h

    def viewDisplayedSize(self) -> (int, int):
        return self.size()

    def mousePressEvent(self, evt):
        x,y = evt.x, evt.y
        ox,oy = self.getViewOffsets()
        index = oy+y
        if oy+y<self._fileBuffer.getLen():
            if x<3:
                if index in self._indexesMark:
                    self._indexesMark.pop(self._indexesMark.index(index))
                else:
                    self._indexesMark.append(index)
                self.marked.emit(self._indexesMark)
            else:
                self._selected = index
                self.selected.emit(self._selected)
            self.update()
            return True
        return False

    @pyTTkSlot(int)
    def selectAndMove(self, line):
        self._selected = line
        ox,_ = self.getViewOffsets()
        self.viewMoveTo(ox, max(0,line-self.height()//2))
        self.update()

    def paintEvent(self):
        ox,oy = self.getViewOffsets()
        for i in range(min(self.height(),self._fileBuffer.getLen()-oy)):
            line = self._fileBuffer.getLine(i+oy)
            if (i+oy) in self._indexesMark:
                color = TTkColor.fg("#00ffff")
                symbol='❥'
            elif (i+oy) in self._indexesSearched:
                color = TTkColor.fg("#ff0000")
                symbol='●'
            else:
                color = TTkColor.fg("#0000ff")
                symbol='○'
            if i+oy == self._selected:
                selectedColor = TTkColor.bg("#008844")
            else:
                selectedColor = TTkColor.RST
                # Check in the filters a matching color
                for filter in TloggCfg.filters:
                    #TTkLog.debug(f"{filter['pattern']} - {line}")
                    if re.search(filter['pattern'],line):
                        #TTkLog.debug("MATCHED")
                        selectedColor = TTkColor.fg(filter['fg'])+TTkColor.bg(filter['bg'])
            self.getCanvas().drawText(pos=(0,i), text=symbol, color=color)
            self.getCanvas().drawText(pos=(2,i), text=line.replace('\t','    ').replace('\n','')[ox:], color=selectedColor )
        if self._indexing is not None:
            self.getCanvas().drawText(pos=(0,0), text=f" [ Indexed: {int(100*self._indexing)}% ] ")

class FileViewerSearch(FileViewer):
    __slots__ = ('_indexes')
    def __init__(self, *args, **kwargs):
        self._indexes = []
        FileViewer.__init__(self, *args, **kwargs)
        self._name = kwargs.get('name' , 'FileViewerSearch' )

    def markIndexes(self, indexes):
        self._indexesMark = indexes
        self._indexes = [i for i in sorted(set(self._indexesSearched+self._indexesMark))]
        ox,oy = self.getViewOffsets()
        self.viewMoveTo(ox,oy)
        self.update()

    def searchedIndexes(self, indexes):
        self._indexesSearched = indexes
        self._indexes = [i for i in sorted(set(self._indexesSearched+self._indexesMark))]
        ox,_ = self.getViewOffsets()
        self.viewMoveTo(ox, 0)
        self.update()

    def viewFullAreaSize(self) -> (int, int):
        if self._indexes is None:
            w = 2+self._fileBuffer.getWidth()
            h = self._fileBuffer.getLen()
        else:
            w = 2+self._fileBuffer.getWidth(self._indexes)
            h = len(self._indexes)
        return w , h

    def mousePressEvent(self, evt):
        x,y = evt.x, evt.y
        ox,oy = self.getViewOffsets()
        index = oy+y
        if index<len(self._indexes):
            if x<3:
                index = self._indexes[oy+y]
                if index in self._indexesMark:
                    self._indexesMark.pop(self._indexesMark.index(index))
                else:
                    self._indexesMark.append(index)
                self.markIndexes(self._indexesMark)
                self.marked.emit(self._indexesMark)
            else:
                self._selected = index
                self.selected.emit(self._indexes[self._selected])
            self.update()
            return True
        return False

    def paintEvent(self):
        ox,oy = self.getViewOffsets()
        if self._indexes:
            allIndexes = self._indexes
            for i in range(min(self.height(),len(allIndexes)-oy)):
                line = self._fileBuffer.getLineDirect(allIndexes[i+oy])
                if allIndexes[i+oy] in self._indexesMark:
                    color = TTkColor.fg("#00ffff")
                    numberColor = TTkColor.bg("#444444")
                    symbol='❥'
                elif allIndexes[i+oy] in self._indexesSearched:
                    color = TTkColor.fg("#ff0000")
                    numberColor = TTkColor.bg("#444444")
                    symbol='●'
                else:
                    color = TTkColor.fg("#0000ff")
                    numberColor = TTkColor.bg("#444444")
                    symbol='○'
                if i+oy == self._selected:
                    selectedColor = TTkColor.bg("#008844")
                else:
                    selectedColor = TTkColor.RST
                    # Check in the filters a matching color
                    for filter in TloggCfg.filters:
                        #TTkLog.debug(f"{filter['pattern']} - {line}")
                        if re.search(filter['pattern'],line):
                            #TTkLog.debug("MATCHED")
                            selectedColor = TTkColor.fg(filter['fg'])+TTkColor.bg(filter['bg'])
                lenLineNumber = len(str(allIndexes[-1]))
                self.getCanvas().drawText(pos=(0,i), text=symbol, color=color)
                # Draw Linenumber
                self.getCanvas().drawText(
                                    pos=(3+lenLineNumber,i),
                                    text=line.replace('\t','    ').replace('\n','')[ox:], color=selectedColor )
                # Draw Line
                self.getCanvas().drawText(pos=(2,i), text=str(allIndexes[i+oy])+" "*lenLineNumber, width=lenLineNumber, color=numberColor)
        if self._indexing is not None:
            self.getCanvas().drawText(pos=(0,0), text=f" [ Indexed: {int(100*self._indexing)}% ] ")

class FileViewerArea(TTkAbstractScrollArea):
    __slots__ = ('_fileView')
    def __init__(self, *args, **kwargs):
        TTkAbstractScrollArea.__init__(self, *args, **kwargs)
        self._name = kwargs.get('name' , 'FileViewer' )
        if 'parent' in kwargs: kwargs.pop('parent')
        self._fileView = kwargs.get('fileView')
        self.setFocusPolicy(TTkK.ClickFocus)
        self.setViewport(self._fileView)
